/* Copyright (C) 2003-2004, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "DataUpdater.hh"
#include "NetworkManager.hh"
#include "../src/version.h"

#include <openssl/ssl.h>
#include <string>
#include <sstream>

extern "C" {
#include <gkrellm2/gkrellm.h>
}

// Play nice with other plugins and keep all our globals in a separate namespace
namespace wmium_gk2 {

#define PLUGIN_PLACEMENT MON_MEM
#define KRELL_HEIGHT 7
#define MONITOR_CONFIG_KEYWORD "wmium"
#define STYLE_NAME "wmium"


#define DEFAULT_HOST "accounts.internode.on.net"
#define DEFAULT_URL "/cgi-bin/padsl-usage"

typedef enum {
  CHART_WEEK = 0, 
  CHART_MONTH,
  CHART_QUARTER,
  CHART_YEAR,
} ChartMode;

//
// Stuff called by gkrellm
//

extern "C" {
static void update_plugin ();
static void create_plugin (GtkWidget *vbox, gint first_create);
GkrellmMonitor *gkrellm_init_plugin ();

static void create_plugin_config_tab (GtkWidget *tab_vbox);
static void save_plugin_config (FILE *f);
static void load_plugin_config (gchar *config_line);
static void apply_plugin_config ();

static void draw_history_chart ();
static gint exposed_widget (GtkWidget *widget, GdkEventExpose *ev);
static void clicked_history_chart (
    GtkWidget *widget,
    GdkEventButton *ev,
    gpointer p);
static void clicked_inode_panel (
    GtkWidget *widget,
    GdkEventButton *ev,
    gpointer p);
static void clicked_usage_panel (
    GtkWidget *widget,
    GdkEventButton *ev,
    gpointer p);

} // end extern "C"

//
// Function prototypes
//

void update_usage_label ();
void update_krell_max ();
void update_krell_value ();
void update_inode_dsl_label ();
void update_history_chart ();
bool is_button_checked (GtkWidget *button);
void update_component_visibility ();


//
// Globals
//
GkrellmMonitor *mon (0);
GkrellmChart *chart (0);
GkrellmChartdata *chart_data (0);
GkrellmChartconfig *chart_config (0);
GkrellmPanel *chart_panel (0);
GkrellmPanel *usage_panel (0);
GkrellmKrell *usage_krell (0);
GkrellmTicks *ticks (0);

gint chart_style_id;
gint meter_style_id;

//
// wmium config globals
//
// For the Options tab
GtkWidget *check_show_usage (0);
GtkWidget *check_show_history_chart (0);
GtkWidget *radio_chart_week (0);
GtkWidget *radio_chart_month (0);
GtkWidget *radio_chart_quarter (0);
GtkWidget *radio_chart_year (0);
GtkWidget *check_enable_meter (0);
GtkWidget *check_option_rollover_date (0);
GtkWidget *check_option_dsl_info (0);

bool show_usage (true);
bool show_history_chart (true);
ChartMode startup_chart_mode (CHART_QUARTER);
ChartMode chart_mode (startup_chart_mode);
bool enable_meter (true);
bool show_dsl_info (false);
bool show_rollover_date (false);

// These are updated by the DataUpdater object
int max_limit (0);
int current_usage (0);
std::string usage ("   ?/   ?");
std::string rollover_date ("R/O:  ?/ ?");
std::string dsl_info (" DSL ????");

std::string chart_peak ("0\\r?");


// For the Settings tab
GtkWidget *username_entry (0);
GtkWidget *password_entry (0);
GtkWidget *hostname_entry (0);
GtkWidget *url_entry (0);

std::string username;
std::string password;
std::string hostname (DEFAULT_HOST);
std::string url (DEFAULT_URL);


//
// Monitor structure
//
GkrellmMonitor wmium_mon = {
  "Internode", // Name shown on config tab
  0,
  create_plugin,
  update_plugin,
  create_plugin_config_tab,
  apply_plugin_config,
  save_plugin_config,
  load_plugin_config,
  MONITOR_CONFIG_KEYWORD,
  0,
  0,
  0,
  PLUGIN_PLACEMENT,
  0,
  0
};


// About tab text
gchar *about_tab_text[] = {
  "<h>About the wmium plugin\n",
  "This software is licensed under the GNU GPLv2.\n",
  "See the file COPYING for license details.\n",
  "\n",
  "Copyright(c) 2003-2005, Johny Mattsson\n",
  "\n",
  "You are currently running version ",
  VERSION,
  ".\n"
};


DataUpdater dataUpdater;
NetworkManager networkManager (dataUpdater);
SampleTransformer historySamples;

//
// Definitions
//

//
// Main functions
//

GkrellmMonitor *gkrellm_init_plugin () {
  mon = &wmium_mon;
  chart_style_id = gkrellm_add_chart_style (mon, STYLE_NAME);
  meter_style_id = gkrellm_add_meter_style (mon, STYLE_NAME);

  ticks = gkrellm_ticks ();

  SSL_library_init ();
  SSL_load_error_strings ();

  return mon;
}


void update_plugin () {
  networkManager.processNetworkData ();

  if (ticks->hour_tick) {
    networkManager.queueNewRequests ();
  }

  /* XXX: Really ugly hack to avoid crashing when used together
     with transparent themes. I have no idea why issuing the
     gkrellm_draw_panel_layers() call causes a SEGV here.
     The SEGV happens for any value [0,4) of the counter - why? */
  static int voodoo;
  if (voodoo < 4) { ++voodoo; return; }

  gkrellm_draw_panel_label (usage_panel);
  gkrellm_draw_panel_layers (usage_panel);

  gkrellm_draw_panel_label (chart_panel);
  gkrellm_draw_panel_layers (chart_panel);

  draw_history_chart ();
}


void create_plugin (GtkWidget *vbox, gint first_create) {
  static GkrellmStyle *panel_style = 0;
  static GkrellmStyle *meter_style = 0;

  if (first_create) {
    usage_panel = gkrellm_panel_new0 ();
    chart = gkrellm_chart_new0 ();
    chart_panel = gkrellm_panel_new0 ();
  }


  if (meter_style) g_free (meter_style);
  meter_style = gkrellm_copy_style (gkrellm_meter_style (meter_style_id));

  gchar *tmp = g_strdup (
      show_rollover_date ? rollover_date.c_str () : usage.c_str ());
  gkrellm_panel_configure (
      usage_panel, // has the meter needle, swaps to rollover date
      tmp,
      meter_style );
  g_free (tmp);

  gkrellm_panel_create (vbox, mon, usage_panel);

  gkrellm_panel_keep_lists (usage_panel, true); // Manually manage krell list


  if (usage_krell) { // Clean up manually so we don't leak memory
    gkrellm_remove_krell (usage_panel, usage_krell);
    gkrellm_destroy_krell (usage_krell);
  }
  
  usage_krell = gkrellm_create_krell (
      usage_panel,
      gkrellm_krell_meter_piximage (meter_style_id),
      meter_style );

  gkrellm_monotonic_krell_values (usage_krell, FALSE);

  gkrellm_set_krell_full_scale (usage_krell, 100, 1);


  gkrellm_chart_create (vbox, mon, chart, &chart_config);

  chart_data = gkrellm_add_default_chartdata (chart, "history");

  gkrellm_monotonic_chartdata (chart_data, FALSE);

  gkrellm_set_draw_chart_function (chart, draw_history_chart, NULL);

  gkrellm_alloc_chartdata (chart);


  if (panel_style) g_free (panel_style);
  panel_style = gkrellm_copy_style (gkrellm_meter_style (meter_style_id));

  tmp = g_strdup (show_dsl_info ? dsl_info.c_str () : "Internode");
  gkrellm_panel_configure (
      chart_panel,
      tmp,
      panel_style );
  g_free (tmp);

  gkrellm_panel_create (vbox, mon, chart_panel);

  update_usage_label ();
  update_inode_dsl_label ();
  update_history_chart ();

  update_component_visibility ();

  if (first_create) {
    g_signal_connect (
        G_OBJECT (usage_panel->drawing_area),
        "expose_event",
        G_CALLBACK (exposed_widget),
        NULL );
    g_signal_connect (
        G_OBJECT (usage_panel->drawing_area),
        "button_press_event",
        G_CALLBACK (clicked_usage_panel),
        NULL);
    g_signal_connect (
        G_OBJECT (chart->drawing_area),
        "expose_event",
        G_CALLBACK (exposed_widget),
        NULL);
    g_signal_connect (
        G_OBJECT (chart->drawing_area),
        "button_press_event",
        G_CALLBACK (clicked_history_chart),
        NULL);
    g_signal_connect (
        G_OBJECT (chart_panel->drawing_area),
        "expose_event",
        G_CALLBACK (exposed_widget),
        NULL);
    g_signal_connect (
        G_OBJECT (chart_panel->drawing_area),
        "button_press_event",
        G_CALLBACK (clicked_inode_panel),
        NULL);

    networkManager.setLoginDetails (&username, &password, &hostname, &url);
    networkManager.queueNewRequests ();
  }
  else {
    draw_history_chart ();
  }
}

//
// Configuration handlers
//

void create_plugin_config_tab (GtkWidget *tab_vbox) {
  GtkWidget *tabs;
  GtkWidget *options_tab;
  GtkWidget *settings_tab;
  GtkWidget *about_tab;
  GtkWidget *about_text;
  GtkWidget *hbox;
  GtkWidget *vbox;
  GtkWidget *label;

  // Prepare tab setup
  tabs = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK(tabs), GTK_POS_TOP);
  gtk_box_pack_start (GTK_BOX(tab_vbox), tabs, TRUE, TRUE, 0);

  // Create the tabs
  options_tab = gkrellm_gtk_framed_notebook_page (tabs, "Options");
  settings_tab = gkrellm_gtk_framed_notebook_page (tabs, "Settings");
  about_tab = gkrellm_gtk_framed_notebook_page (tabs, "About");

  // Fill in the Options tab
  vbox = gkrellm_gtk_framed_vbox (
      options_tab,
      "Component visibility",
      2,
      FALSE,
      0,
      2);
  gkrellm_gtk_check_button_connected (
      vbox,
      &check_show_usage,
      show_usage,
      FALSE, // expand
      FALSE, // fill
      0, // pad
      0, // callback
      0, // gpointer
      "Show usage/rollover info panel");
  gkrellm_gtk_check_button_connected (
      vbox,
      &check_enable_meter,
      enable_meter,
      FALSE,
      FALSE,
      0,
      0,
      0,
      "Show meter needle on usage panel");
  gkrellm_gtk_check_button_connected (
      vbox,
      &check_show_history_chart,
      show_history_chart,
      FALSE,
      FALSE,
      0,
      0,
      0,
      "Show history chart");

  vbox = gkrellm_gtk_framed_vbox (
      options_tab,
      "Startup chart mode",
      2,
      FALSE,
      0,
      2);

  radio_chart_week = gtk_radio_button_new_with_label (0, "Week");
  radio_chart_month = gtk_radio_button_new_with_label_from_widget (
      GTK_RADIO_BUTTON(radio_chart_week),
      "Month");
  radio_chart_quarter = gtk_radio_button_new_with_label_from_widget (
      GTK_RADIO_BUTTON(radio_chart_month),
      "Quarter");
  radio_chart_year = gtk_radio_button_new_with_label_from_widget (
      GTK_RADIO_BUTTON(radio_chart_quarter),
      "Year");

  hbox = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(hbox), radio_chart_week, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(hbox), radio_chart_quarter, TRUE, TRUE, 0);
  hbox = gtk_hbox_new (TRUE, 0);
  gtk_box_pack_start (GTK_BOX(vbox), hbox, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(hbox), radio_chart_month, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX(hbox), radio_chart_year, TRUE, TRUE, 0); 

  switch (startup_chart_mode) {
    case CHART_WEEK:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio_chart_week), TRUE);
      break;
    case CHART_MONTH:
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio_chart_month), TRUE);
      break;
    case CHART_QUARTER:
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(radio_chart_quarter),TRUE);
      break;
    case CHART_YEAR: 
      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(radio_chart_year), TRUE);
      break;
  }

  vbox = gkrellm_gtk_framed_vbox (
      options_tab,
      "Startup panel modes",
      2,
      FALSE,
      0,
      2);
  gkrellm_gtk_check_button_connected (
      vbox,
      &check_option_rollover_date,
      show_rollover_date,
      FALSE,
      FALSE,
      0,
      0,
      0,
      "Show rollover date instead of usage");
  gkrellm_gtk_check_button_connected (
      vbox,
      &check_option_dsl_info,
      show_dsl_info,
      FALSE,
      FALSE,
      0,
      0,
      0,
      "Show DSL information instead of 'Internode' label");


  // Fill in the Settings tab

  // Normal settings
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(settings_tab), hbox, TRUE, TRUE, 0);
  label = gtk_label_new ("Mandatory settings:");
  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(settings_tab), hbox, TRUE, TRUE, 0);
  label = gtk_label_new ("Username");
  gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, TRUE, 0);

  username_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY(username_entry), username.c_str ());
  gtk_box_pack_start (GTK_BOX(hbox), username_entry, TRUE, TRUE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(settings_tab), hbox, TRUE, TRUE, 0);
  label = gtk_label_new ("Password");
  gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, TRUE, 0);

  password_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY(password_entry), password.c_str ());
  gtk_entry_set_visibility (GTK_ENTRY(password_entry), FALSE);
  gtk_box_pack_start (GTK_BOX(hbox), password_entry, TRUE, TRUE, 0);

  // Advanced settings
  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(settings_tab), hbox, TRUE, TRUE, 0);
  label = gtk_label_new ("Advanced settings (leave blank for defaults):");
  gtk_box_pack_start (GTK_BOX(hbox), label, FALSE, FALSE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(settings_tab), hbox, TRUE, TRUE, 0);
  label = gtk_label_new ("Hostname");
  gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, TRUE, 0);
  
  hostname_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY(hostname_entry), hostname.c_str ());
  gtk_box_pack_start (GTK_BOX(hbox), hostname_entry, TRUE, TRUE, 0);

  hbox = gtk_hbox_new (FALSE, 0);
  gtk_box_pack_start (GTK_BOX(settings_tab), hbox, TRUE, TRUE, 0);
  label = gtk_label_new ("CGI script");
  gtk_box_pack_start (GTK_BOX(hbox), label, TRUE, TRUE, 0);
  
  url_entry = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY(url_entry), url.c_str ());
  gtk_box_pack_start (GTK_BOX(hbox), url_entry, TRUE, TRUE, 0);


  // Fill in the About tab
  about_text = gkrellm_gtk_scrolled_text_view (about_tab,
      NULL,
      GTK_POLICY_AUTOMATIC,
      GTK_POLICY_AUTOMATIC);
  for (unsigned i = 0; i < sizeof (about_tab_text) / sizeof (gchar *); ++i) {
    gkrellm_gtk_text_view_append (about_text, about_tab_text[i]);
  }
}

void save_plugin_config (FILE *f) {
  fprintf (f, "%s username %s\n", MONITOR_CONFIG_KEYWORD, username.c_str ());
  fprintf (f, "%s password %s\n", MONITOR_CONFIG_KEYWORD, password.c_str ());
  fprintf (f, "%s hostname %s\n", MONITOR_CONFIG_KEYWORD, hostname.c_str ());
  fprintf (f, "%s url %s\n", MONITOR_CONFIG_KEYWORD, url.c_str ());
  fprintf (f, "%s showusage %d\n", MONITOR_CONFIG_KEYWORD, show_usage);
  fprintf (f, "%s showchart %d\n", MONITOR_CONFIG_KEYWORD, show_history_chart);
  fprintf (f, "%s chartmode %d\n", MONITOR_CONFIG_KEYWORD, startup_chart_mode);
  fprintf (f, "%s meter %d\n", MONITOR_CONFIG_KEYWORD, enable_meter);
  fprintf (f, "%s rollover %d\n", MONITOR_CONFIG_KEYWORD, show_rollover_date);
  fprintf (f, "%s dslinfo %d\n", MONITOR_CONFIG_KEYWORD, show_dsl_info);
  gkrellm_save_chartconfig (f, chart_config, MONITOR_CONFIG_KEYWORD, NULL);
}

void load_plugin_config (gchar *config_line) {
  gchar keyword[32], value[512];

  value[sizeof (value) -1] = 0; // always ensure it is zero terminated

  if (sscanf (config_line, "%31s %511[^\n]", keyword, value) != 2) return;

  if (strcmp ("username", keyword) == 0) username = value;
  else if (strcmp ("password", keyword) == 0) password = value;
  else if (strcmp ("hostname", keyword) == 0) hostname = value;
  else if (strcmp ("url", keyword) == 0) url = value;
  else if (strcmp (GKRELLM_CHARTCONFIG_KEYWORD, keyword) == 0) {
    gkrellm_load_chartconfig (&chart_config, value, 1);
  }
  else if (strcmp ("showusage", keyword) == 0) show_usage = atoi (value);
  else if (strcmp ("showchart", keyword) == 0) show_history_chart = atoi(value);
  else if (strcmp ("chartmode", keyword) == 0) {
    startup_chart_mode = (ChartMode)atoi (value);
    chart_mode = startup_chart_mode;
  }
  else if (strcmp ("meter", keyword) == 0) enable_meter = atoi (value);
  else if (strcmp ("rollover", keyword) == 0) show_rollover_date = atoi (value);
  else if (strcmp ("dslinfo", keyword) == 0) show_dsl_info = atoi (value);
}

void apply_plugin_config () {
  // Options tab
  if (check_show_usage)
    show_usage = is_button_checked (check_show_usage);

  if (check_show_history_chart)
    show_history_chart = is_button_checked (check_show_history_chart);

  if (radio_chart_week && radio_chart_month && radio_chart_quarter && 
    radio_chart_year) {

     if (is_button_checked (radio_chart_week)) startup_chart_mode = CHART_WEEK;
     else if (is_button_checked (radio_chart_month))
       startup_chart_mode = CHART_MONTH;
     else if (is_button_checked (radio_chart_quarter))
       startup_chart_mode = CHART_QUARTER;
     else if (is_button_checked (radio_chart_year))
       startup_chart_mode = CHART_YEAR;
     else startup_chart_mode = CHART_QUARTER; // This should never be needed
  }

  if (check_enable_meter)
    enable_meter = is_button_checked (check_enable_meter);

  if (check_option_rollover_date)
    show_rollover_date = is_button_checked (check_option_rollover_date);

  if (check_option_dsl_info)
    show_dsl_info = is_button_checked (check_option_dsl_info);

  update_component_visibility ();

  // Settings tab
  std::string temp;
  bool changes_made (false);

  if (username_entry && password_entry && hostname_entry && url_entry) {
    // Only attempt to apply if all the text entry boxes actually exist

    temp = gtk_entry_get_text (GTK_ENTRY(username_entry));
    if (username != temp) changes_made = true;
    username = temp;
    if (username.find ("@") == std::string::npos) {
      // Only username entered, append @internode.on.net
      // This will ensure that history data is obtained correctly
      username += "@internode.on.net";
    }

    temp = gtk_entry_get_text (GTK_ENTRY(password_entry));
    if (password != temp) changes_made = true;
    password = temp;

    temp = gtk_entry_get_text (GTK_ENTRY(hostname_entry));
    if (hostname != temp) changes_made = true;
    hostname = temp.length () ? temp : DEFAULT_HOST;

    temp = gtk_entry_get_text (GTK_ENTRY(url_entry));
    if (url != temp) changes_made = true;
    url = temp.length () ? temp : DEFAULT_URL;
  }

  if (changes_made) {
    networkManager.setLoginDetails (&username, &password, &hostname, &url);
    networkManager.queueNewRequests ();
  }
}

//
// Drawing functions
//

void draw_history_chart () {
  gkrellm_draw_chartdata (chart);
  gchar *peak = g_strdup (chart_peak.c_str ());
  gkrellm_draw_chart_text (chart, chart_style_id, peak);
  g_free (peak);
  gkrellm_draw_chart_to_screen (chart);
}

gint exposed_widget (GtkWidget *widget, GdkEventExpose *ev) {
  gdk_draw_pixmap (widget->window,
      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
      chart->pixmap,
      ev->area.x, ev->area.y,
      ev->area.x, ev->area.y,
      ev->area.width, ev->area.height);
  return FALSE;
}

//
// Click handlers
//

void clicked_history_chart (GtkWidget *, GdkEventButton *, gpointer) {
  switch (chart_mode) {
    case CHART_WEEK: chart_mode = CHART_MONTH; break;
    case CHART_MONTH: chart_mode = CHART_QUARTER; break;
    case CHART_QUARTER: chart_mode = CHART_YEAR; break;
    case CHART_YEAR: chart_mode = CHART_WEEK; break;
  }

  update_history_chart ();
  
  draw_history_chart ();
}

void clicked_inode_panel (GtkWidget *, GdkEventButton *, gpointer) {
  show_dsl_info = !show_dsl_info;
  
  update_inode_dsl_label ();

  gkrellm_draw_panel_label (chart_panel);
  gkrellm_draw_panel_layers (chart_panel);
}

void clicked_usage_panel (GtkWidget *, GdkEventButton *, gpointer) {
  show_rollover_date = !show_rollover_date;

  update_usage_label ();
  gkrellm_draw_panel_label (usage_panel);
  gkrellm_draw_panel_layers (usage_panel);
}

//
// Data updaters
//

void update_usage_label () {
  g_free (usage_panel->label->string);

  usage_panel->label->string = g_strdup (
      show_rollover_date ? rollover_date.c_str () : usage.c_str () );
}

void update_krell_max () {
  gkrellm_set_krell_full_scale (usage_krell, max_limit, 1);
}

void update_krell_value () {
  gkrellm_update_krell (usage_panel, usage_krell, current_usage);
}

void update_inode_dsl_label () {
  g_free (chart_panel->label->string);

  chart_panel->label->string = g_strdup (
      show_dsl_info ? dsl_info.c_str () : "Internode");
}

void update_history_chart () {
  std::string letter ("?");
  int include_size (0);

  historySamples.setOutputVectorSize (gkrellm_chart_width ());
  historySamples.setNormalizationLevel (
      gkrellm_get_chartconfig_height (chart_config));

  switch (chart_mode) {
    case CHART_WEEK: include_size = 7; letter = "W"; break;
    case CHART_MONTH: include_size = 30; letter = "M"; break;
    case CHART_QUARTER: include_size = 90; letter = "Q"; break;
    case CHART_YEAR: include_size = 365; letter = "Y"; break;
  }

  historySamples.setIncludeSize (include_size);

  gkrellm_reset_chart (chart);

  SampleVector out (historySamples.getOutputVector ());
  SampleVector::iterator iter (out.begin ());
  SampleVector::iterator end (out.end ());
  for (;iter != end; ++iter)
    gkrellm_store_chartdata (chart, 0, static_cast<gulong> (*iter));
    

  int peak_value = static_cast<int> (historySamples.getPeakValue ());
  std::ostringstream str;
  str << "PK:" <<  peak_value << "\\r" << letter;
  chart_peak = str.str ();
}

//
// Config related helpers
//

bool is_button_checked (GtkWidget *button) {
  return gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(button));
}

void update_component_visibility () {
  if (show_history_chart) gkrellm_chart_show (chart, true);
  else gkrellm_chart_hide (chart, true);

  if (show_usage) gkrellm_panel_show (usage_panel);
  else gkrellm_panel_hide (usage_panel);

  if (enable_meter) {
    gkrellm_insert_krell (usage_panel, usage_krell, false);
    update_krell_max ();
    update_krell_value ();
  }
  else {
    gkrellm_remove_krell (usage_panel, usage_krell);
  }
}


} // end namespace
