/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "FDHolder.hh"

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>

/**
 * Initializes an unassigned FDHolder.
 *
 * Operations on an FDHolder in this state will fail or indicate nothingness.
 */
FDHolder::FDHolder () : m_fd (-1), m_read (false), m_write (false) {
  // Empty body
}

/**
 * Initialize the FDHolder to the given file descriptor.
 *
 * @param fd The file descriptor to tie this FDHolder to.
 */
FDHolder::FDHolder (int fd) : m_fd (fd), m_read (false), m_write (false) {
  // Empty body
}

/**
 * Destroys the FDHolder, but does NOT close the file descriptor assigned to it.
 */
FDHolder::~FDHolder () {
  // do nothing, in particular, do NOT close the FD!
  // we don't do any resource management (i.e. think about copy ctor etc)
  // so we don't actually own the FD, we just hold it
}


/**
 * Returns the actual file descriptor that is beeing held.
 *
 * @returns The filedescriptor, or -1 if no descriptor is assigned.
 */
int FDHolder::getFD () {
  return m_fd;
}

/**
 * Assigns a new file descriptor to this holder.
 *
 * The old file descriptor (if any) is NOT closed.
 *
 * @param fd The new file descriptor to assign to this holder.
 */
void FDHolder::resetFD (int fd) {
  m_fd = fd;
  m_read = false;
  m_write = false;
}

/**
 * Returns whether this holder has a descriptor assigned to it or not.
 *
 * @returns The file descriptor, or -1 if no descriptor is assigned to it.
 */
bool FDHolder::isValidFD () const {
  return m_fd != -1;
}

/**
 * Returns whether someone has expressed an interest in read()s being possible
 * on the held descriptor.
 *
 * @returns True if "read possible" should be checked for, false if not.
 */
bool FDHolder::wantRead () const {
  return m_read;
}

/**
 * Returns whether someone has expressed an interest in write()s being possible
 * on the held descriptor.
 *
 * @returns True if "write possible" should be checked for, false if not.
 */
bool FDHolder::wantWrite () const {
  return m_write;
}

/**
 * Checks whether the held file descriptor is possible to read from.
 *
 * @returns True if it is possible to read (without blocking) from the held
 *     file descriptor, false if not.
 */
bool FDHolder::canRead () const {
  if (!isValidFD ()) return false;

  fd_set fds;
  FD_ZERO (&fds);
  FD_SET (m_fd, &fds);

  timeval tv;
  tv.tv_sec = 0;
  tv.tv_usec = 0;
  int ret;
  do {
    ret = select (m_fd + 1, &fds, 0, 0, &tv);
  } while (ret == -1 && errno == EINTR);

  if (ret == 1) return true;
  else return false;
}

/**
 * Checks whether the held file descriptor is possible to write to.
 *
 * @returns True if the held file descriptor can be written to (without
 *     blocking), false if not.
 */
bool FDHolder::canWrite () const {
  if (!isValidFD ()) return false;

  fd_set fds;
  FD_ZERO (&fds);
  FD_SET (m_fd, &fds);

  timeval tv;
  tv.tv_sec = 0;
  tv.tv_usec = 0;
  int ret;
  do {
    ret = select (m_fd + 1, 0, &fds, 0, &tv);
  } while (ret == -1 && errno == EINTR);

  if (ret == 1) return true;
  else return false;
}

/**
 * Checks if the held file descriptor has an event ready to be processed.
 *
 * @returns True if the file descriptor has been marked as "read wanted" and
 *     read is now possible, or if "write wanted" and write is now possible.
 *     False otherwise.
 */
bool FDHolder::hasEvent () const {
  if (!isValidFD ()) return false;

  if (wantRead () && canRead ()) return true;
  if (wantWrite () && canWrite ()) return true;
  return false;
}

/**
 * Sets the state of the "read wanted" flag to signal whether the held
 * file descriptor should be checked for read() possibility.
 *
 * @param yes New state of the "read wanted" flag.
 */
void FDHolder::setReadWanted (bool yes) {
  m_read = yes;
}

/**
 * Sets the state of the "write wanted" flag to signal whether the held
 * file descriptor should be checked for write() possibility.
 *
 * @param yes New state of the "write wanted" flag.
 */
void FDHolder::setWriteWanted (bool yes) {
  m_write = yes;
}

