/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "FDSet.hh"

#include <sys/types.h>
#include <sys/time.h>
#include <unistd.h>
#include <errno.h>
#include <algorithm>

/**
 * Initializes an empty FDSet.
 */
FDSet::FDSet () {
  // Empty body
}

/**
 * Destroys the FDSet.
 */
FDSet::~FDSet () {
  // Empty body
}

/**
 * Adds the given FDHolder to the set.
 *
 * @param holder The FDHolder to add.
 * @see FDHolder
 */
void FDSet::addFDHolder (FDHolder& holder) {
  m_fds.push_back (&holder);
}

/**
 * Removes the FDHolder from the set.
 *
 * @param holder The FDHolder to remove.
 * @see FDHolder
 */
void FDSet::removeFDHolder (FDHolder& holder) {
  m_fds.erase (find (m_fds.begin (), m_fds.end (), &holder));
}

/**
 * Checks for the existence of any network events, and optionally blocks
 * until one is received.
 *
 * A network event is when one or more of the FDs in the set have the
 * requested state (i.e. read or write is possible).
 *
 * @returns True if the wait has been satisfied, or if it was interrupted by
 *     a signal. False in the case of any other error.
 */
bool FDSet::checkForEvent (bool block) {
  fd_set readfds;
  fd_set writefds;

  FD_ZERO(&readfds);
  FD_ZERO(&writefds);

  bool haveSomethingToWaitFor = false;
  int maxfd = 0;

  FDVector::iterator iter = m_fds.begin ();
  FDVector::iterator end = m_fds.end ();

  // Note: All these (*iter) should probably be done away with for efficiency
  // reasons. Dereference once and cache the value would be better.
  for (;iter != end; ++iter) {
    if ((*iter)->wantRead ()) {
      FD_SET((*iter)->getFD (), &readfds);
      haveSomethingToWaitFor = true;
      if ((*iter)->getFD () > maxfd) maxfd = (*iter)->getFD ();
    }
    if ((*iter)->wantWrite ()) {
      FD_SET((*iter)->getFD (), &writefds);
      haveSomethingToWaitFor = true;
      if ((*iter)->getFD () > maxfd) maxfd = (*iter)->getFD ();
    }
  }

  if (!haveSomethingToWaitFor) return false;

  timeval waitTime; // Set up for 0.000 second wait time
  waitTime.tv_sec = 0;
  waitTime.tv_usec = 0;

  int ret = select (maxfd +1, &readfds, &writefds, 0, block ? 0 : &waitTime);
  if (ret >= 0) return true;
  else if (errno == EINTR) return true;
  else return false;
}

