/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "HistoryDataHandler.hh"
#include "PanelHandler.hh"
#include "ERegEx.hh"
#include "SampleTransformer.hh"
#include "SimpleCalendar.hh"
#include <sstream>
#include <iostream>

using std::cerr;
using std::endl;
using std::istringstream;
using std::string;

/**
 * Initializes the object.
 *
 * @param panel The PanelHandler to use for communicating information to the
 *     history graph panel.
 * @see PanelHandler
 */
HistoryDataHandler::HistoryDataHandler (PanelHandler& panel)
  : m_panel (panel)
{
  // Empty body
}

/**
 * Destroys the object.
 */
HistoryDataHandler::~HistoryDataHandler () {
  // Empty body
}

/**
 * Interprets the HTTP response (including header).
 *
 * Attempts to validate the response as far as possible, and may print
 * diagnostics to <code>cerr</code>. If a format error is discovered in the
 * actual history data, the status panel's text is also updated to notify
 * the user that something is really amiss.
 *
 * @param data The full HTTP response string to be processed.
 */
void HistoryDataHandler::processData (string& data) {
  istringstream extractor (data);
  string resultCode;
  if (!getline (extractor, resultCode)) {
    cerr << "Error: historycheck: HTTPS server returned malformed header!\n";
    return;
  }

  if (resultCode.find ("200") == string::npos) {
    cerr << "Error: historycheck: HTTPS server reports failure: ";
    cerr << resultCode << endl;
    return;
  }

  string line;
  while (getline (extractor, line)) {
    if (!line.length ()) break; // Respond to LF as blank line

    // Also allow CRLF as blank line
    if ((line.length () == 1) && (line == "\r")) break;
  }

  if (!getline (extractor, line)) { // No body??
    cerr << "Warning: historycheck: HTTPS server returned no data! "
         << "Assuming no historical data available.\n";
  }

  // Regex to check the returned lines against to ensure we get the data
  // in the correct format before we attempt to use it
  ERegEx expectedFormat ("[0-9]{6} -?[0-9]+(\\.[0-9]+)?");

  // Note that this code depends on history data being returned in
  // chronological order.
  SampleVector historyData;
  SimpleCalendar *lastDate (0);
  while (line.length () > 1) {
    if (!expectedFormat.isMatch (line)) {
      cerr << "Error: Unrecognized format of history data: " << line << endl;
      m_panel.getStatusReceiver ().setErrorText (" ERROR! ");
      m_panel.reDraw ();
      return;
    }

    istringstream usage (line);
    unsigned long date;
    usage >> date;

    // XXX: Do somewhat dodgy year expansion:
    //  Dates before xx90 are considered to be in the 2000 spectrum,
    //  whereas years xx90-xx99 are considered to be part of the 1900 range.
    //  The cutoff point is based on the fact that Internode goes back to 1991.
    if (date < 90 * 10000) date += 2000 * 10000;
    else date += 1900 * 10000;

    if (!lastDate) lastDate = new SimpleCalendar (date); // First returned entry
    else {
      lastDate->advanceDays (1); // Expect sequential dates

      // Ensure we compensate for missed days (i.e. days with 0 download)
      while (lastDate->isoDate () != date) {
        historyData.push_back (0);
        lastDate->advanceDays (1);
      }
    }

    double value;
    usage >> value;
    historyData.push_back (value);

    if (!getline (extractor, line)) break;
  }
  delete lastDate;

  // Calculate rolling usage
  double rollingUsage = 0;
  SampleVector::reverse_iterator ri = historyData.rbegin ();
  SampleVector::reverse_iterator end = historyData.rend ();
  for (int i = 0; i < 7; ++i) {
    if (ri == end) break; // if we have less than 7 days worth of data

    rollingUsage += *ri++;
  }
  // Round the total before using
  m_panel.getStatusReceiver ().setRollingUsage (
      static_cast<int> (rollingUsage + 0.5));

  m_panel.getHistoryReceiver ().setGraphData (historyData);
  m_panel.reDraw ();
}

