/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "HistoryPainter.hh"
#include "xpm/field_7ch.xpm"
#include "xpm/field_1ch.xpm"
#include "xpm/field_graph.xpm"
#include "xpm/graphmask.xbm"
#include <sstream>
#include <iomanip> 
#include <algorithm>

using std::ostringstream;
using std::setfill;
using std::setw;

/**
 * Initializes the HistoryPainter object and ties it to a Display and
 * WMDockWindow.
 *
 * @param dpy The X Diplay to tie this object to.
 * @param window The WMDockWindow to use for output.
 */
HistoryPainter::HistoryPainter (Display& dpy, WMDockWindow& window)
  : m_display (dpy), m_window (window), m_charset (dpy),
    m_peak (dpy, window, m_charset, 4, 4, field_7ch_xpm),
    m_modeIndicator (dpy, window, m_charset, 51, 4, field_1ch_xpm),
    m_graph (dpy, window, 4, 18, field_graph_xpm),
    m_mask (), m_dataTransformer (), m_mode (GRAPHWEEK)
{
  m_mask = XCreateBitmapFromData (&dpy, XDefaultRootWindow (&dpy),
     reinterpret_cast<const char *> (graphmask_bits), graphmask_width,
     graphmask_height);

  m_dataTransformer.setOutputVectorSize (m_graph.getGraphWidth ());
  m_dataTransformer.setNormalizationLevel (m_graph.getGraphHeight (), 95);

  updateMode ();

  m_peak.setText ("PK:    ");
}

/**
 * Destroys the object and frees associated resources.
 */
HistoryPainter::~HistoryPainter () {
  XFreePixmap (&m_display, m_mask);
}

/**
 * Sets the graph mode to use for the display.
 *
 * @param mode The graph mode to use.
 */
void HistoryPainter::setMode (GraphMode mode) {
  m_mode = mode;

  updateMode ();
  updatePeakValue ();
}

/**
 * Checks the graph mode currently in use.
 *
 * @returns The current graph mode.
 */
GraphMode HistoryPainter::getMode () const {
  return m_mode;
}

/**
 * Sets new, untransformed, history data to display.
 *
 * @param data The new history data to use. This will be automatically copied
 *     and transformed into the right format ready for graph usage.
 */
void HistoryPainter::setGraphData (const SampleVector& data) {
  m_dataTransformer.setSampleVector (data);
  m_graph.setGraphData (m_dataTransformer.getOutputVector ());

  updatePeakValue ();
}

/**
 * Updates the display string for the peak value found.
 */
void HistoryPainter::updatePeakValue () {
  double maxval = m_dataTransformer.getPeakValue ();

  ostringstream oss;
  oss << "PK:" << setw (4) << setfill (' ') << static_cast<int> (maxval);
  m_peak.setText (oss.str ());
}

/**
 * Checks whether a click coordinate is withing the area that indicates that
 * a graph mode change is desired.
 *
 * @param x The X coordinate, with the WMDockWindow as reference object.
 * @param y The Y coordinate, with the WMDockWindow as reference object.
 * @returns True if the coordinate is within the mode change area, false if not.
 */
bool HistoryPainter::isInModeChangeArea (int x, int y) {
  // XXX: should obtain these values from the XPM instead
  if (x >= 51 && x <= 59 && y >= 4 && y <= 15) return true;
  else return false;
}

/**
 * Reapplies the shape mask to the WMDockWindow.
 */
void HistoryPainter::resetMask () {
  m_window.setMask (m_mask);
}

/**
 * Redraws the history panel and issues an XFlush.
 */
void HistoryPainter::reDraw () {
  m_modeIndicator.reDraw ();
  m_peak.reDraw ();
  m_graph.reDraw ();

  XFlush (&m_display);
}

/**
 * Modifies the sample transformer and mode indicator field to match
 * the current graph mode.
 */
void HistoryPainter::updateMode () {
  switch (m_mode) {
    case GRAPHWEEK: m_dataTransformer.setIncludeSize (7);
        m_modeIndicator.setText ("W"); break;
    case GRAPHMONTH: m_dataTransformer.setIncludeSize (30);
        m_modeIndicator.setText ("M"); break;
    case GRAPHQUARTER: m_dataTransformer.setIncludeSize (90);
        m_modeIndicator.setText ("Q"); break;
    case GRAPHYEAR: m_dataTransformer.setIncludeSize (365);
        m_modeIndicator.setText ("Y"); break;
  }
  m_graph.setGraphData (m_dataTransformer.getOutputVector ());
}

