/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "PanelSelector.hh"

/**
 * Initializes the PanelSelector object and creates the Status and History
 * panels.
 *
 * @param dpy The X Display to tie the panels to.
 * @param window The WMDockWindow to tie the panels to.
 * @param status Whether the Status panel should be the active panel or not.
 */
PanelSelector::PanelSelector (Display& dpy, WMDockWindow& window, bool status)
  : m_statusPainter (dpy, window), m_historyPainter (dpy, window),
    m_statusActive (status)
{
  // Empty body
  m_statusPainter.resetMask ();
}

/**
 * Destroys the object.
 */
PanelSelector::~PanelSelector () {
  // Empty body
}

/**
 * Changes the active panel to that specified.
 *
 * Does not issue an XFlush, but does apply the shape mask for the selected
 * panel.
 *
 * @param choice The new panel to be active. Either STATUSPANEL or HISTORYPANEL.
 */
void PanelSelector::selectPanel (PanelChoice choice) {
  switch (choice) {
    case STATUSPANEL: m_statusActive = true; break;
    case HISTORYPANEL: m_statusActive = false; break;
  }
  if (m_statusActive) m_statusPainter.resetMask ();
  else m_historyPainter.resetMask ();
}

/** 
 * Selects the next panel in order.
 *
 * As there are only two different panels, in effect toggles between the two.
 */
void PanelSelector::selectNextPanel () {
  m_statusActive = !m_statusActive;
  if (m_statusActive) m_statusPainter.resetMask ();
  else m_historyPainter.resetMask ();
}

/**
 * Redraws the active panel.
 *
 * In effect also issues the XFlush command.
 */
void PanelSelector::reDraw () {
  if (m_statusActive) m_statusPainter.reDraw ();
  else m_historyPainter.reDraw ();
}

/**
 * Processes a mouse click at the given coordinates.
 *
 * If the active panel is the status panel, the history panel is made active.
 * If the history panel is active, a check is made to see if the click is
 * an attempt to change the graph mode. If not, the status panel is made the
 * active panel. Else the graph mode is changed, either forward, or backward
 * (if <code>reverse</code> is true).
 *
 * @param The X coordinate, relative to the WMDockWindow.
 * @param The Y coordinate, relative to the WMDockWindow.
 * @param reverse True if the graph mode should be cycled backwards, false if
 *     if should be cycled forwards. This value has no effect unless a graph
 *     mode change is actually requested (see above for details on when that
 *     occurs).
 */
void PanelSelector::processClick (int x, int y, bool reverse) {
  if (m_statusActive) selectNextPanel ();
  else {
    // Check if we should just change graph mode
    if (m_historyPainter.isInModeChangeArea (x, y)) {
      // Yep, find next mode (rotation)
      GraphMode mode = m_historyPainter.getMode ();
      switch (mode) {
        case GRAPHWEEK: mode = reverse ? GRAPHYEAR : GRAPHMONTH; break;
        case GRAPHMONTH: mode = reverse ? GRAPHWEEK : GRAPHQUARTER; break;
        case GRAPHQUARTER: mode = reverse ? GRAPHMONTH : GRAPHYEAR; break;
        case GRAPHYEAR: mode = reverse ? GRAPHQUARTER : GRAPHWEEK; break;
      }
      m_historyPainter.setMode (mode);
    }
    else selectNextPanel ();
  }

  reDraw ();
}

/**
 * Returns a reference to the HistoryReceiver.
 *
 * This object is in fact a HistoryPainter. To access the full object, use
 * getHistoryPainter.
 *
 * @returns The HistoryReceiver object.
 * @see HistoryReceiver
 * @see getHistoryPainter
 */
HistoryReceiver& PanelSelector::getHistoryReceiver () {
  return m_historyPainter;
}

/**
 * Returns a reference to the StatusReceiver.
 *
 * This object is in fact a StatusPainter. To access the full object, use
 * getStatusPainter.
 *
 * @returns The StatusReceiver object.
 * @see StatusReceiver
 * @see getStatusPainter
 */
StatusReceiver& PanelSelector::getStatusReceiver () {
  return m_statusPainter;
}

/**
 * Returns a reference to the HistoryPainter.
 *
 * @returns The HistoryPainter object.
 * @see HistoryPainter
 */
HistoryPainter& PanelSelector::getHistoryPainter () {
  return m_historyPainter;
}

/**
 * Returns a reference to the StatusPainter.
 *
 * @returns The StatusPainter object.
 * @see StatusPainter
 */
StatusPainter& PanelSelector::getStatusPainter () {
  return m_statusPainter;
}

