/* Copyright (C) 2002-2005, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "Request.hh"

#include <stdexcept>

#include <iostream>

using std::invalid_argument;
using std::string;

/**
 * Initializes the Request object.
 *
 * @param s The SSL client socket to write the request to and retrieve the
 *     response from. Ownership of <code>s</code> transfers to the Request.
 * @param request The HTTP request string in its entirety.
 * @param handler The DataHandler that should be handed the HTTP response.
 * @param snoop Set to true to write data to stderr before handing it off.
 * @exception invalid_argument If <code>s</code> is null, or if
 *     <code>request</code> is an empty string.
 * @see SSLClientSocket
 * @see DataHandler
 */
Request::Request (SSLClientSocket *s,
                  const string& request,
                  DataHandler& handler,
                  bool snoop)
  : m_sock (s), m_request (request), m_handler (handler), m_data (),
    m_snoop (snoop)
{
  if (!s) throw invalid_argument ("request without a socket?!");
  if (!m_request.length ()) throw invalid_argument ("no request in request?!");
}

/**
 * Deletes this Request object and the SSLClientSocket associated with it.
 */
Request::~Request () {
  delete m_sock;
}

/**
 * Returns the FDHolder used by the SSLClientSocket.
 *
 * @returns The FDHolder used by the SSLClientSocket.
 * @see FDHolder
 */
FDHolder& Request::getFDHolder () {
  return m_sock->getFDHolder ();
}

/**
 * Gives the Request object a chance to process any pending network
 * operations (read or write).
 *
 * @return True if the request has been completed, either successfully
 *         or unsuccessfully (due to an I/O error). False if the request
 *         is still in progress after the return of this call.
 */
bool Request::processNetworkEvent () {
  // See if we have more of the request to send
  if (m_request.length ()) {
    int ret = m_sock->write (m_request);

    if (ret > 0) {
      m_request = m_request.substr (ret, string::npos);
      if (m_request.length ()) { // Still got more to write
        m_sock->getFDHolder ().setWriteWanted (true);
      }
    }
    else if (ret == -1) return true; // Error, flag as completion
    else return false; // Couldn't write, so just wait
  }

  // Request sent, reading the response now
  int ret = 0;
  do {
    ret = m_sock->read (m_data);
    if (ret == -1) {
      if (m_sock->isReadEOF ()) {
        if (m_snoop) {
          // Possibly write data to stderr if snooping is enabled before
          // the data is handed off for processing
          std::cerr << "Request " << this << ":\n" << m_data << "\n-- end --\n";
        }

        // At end of data, pass it onto the datahandler
        m_handler.processData (m_data);
        return true;
      }
      else return true; // Error, flag as completion
    }
  } while (ret > 0);

  return false;
}

