/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "ResourceFile.hh"
#include <fstream>

using std::ifstream;
using std::string;

/**
 * Associates a filename with the ResourceFile to load resource
 * settings from (using the <code>load()</code> member function).
 *
 * @param filename The filename to read resource settings from later.
 */
ResourceFile::ResourceFile (const string& filename)
  : m_filename (filename), m_entries ()
{
  // empty body
}


/**
 * Destroys the ResourceFile.
 */
ResourceFile::~ResourceFile () {
  m_entries.clear (); // shouldn't be necessary?
}


/**
 * Loads the resource settings from the associated file.
 *
 * If the loading fails the ResourceFile will not contain any resource
 * settings at all.
 *
 * @returns True if resource file is read successfully, false otherwise.
 */
bool ResourceFile::load () {
  m_entries.clear ();

  ifstream is (m_filename.c_str());
  if (!is) return false;

  string line;
  while (getline(is, line)) {
    if (!line.length ()) continue; // allow empty lines
    if (line.length () && line[0] == '#') continue; // allow comments

    string::size_type equalsign = line.find ('=');
    if (equalsign == string::npos) return false; // missing =
    string key = line.substr (0, equalsign);
    string value = line.substr (equalsign +1, string::npos);

    m_entries[key] = value;
  }

  if (is.bad ()) {
    m_entries.clear ();
    return false;
  }

  return true;
}


/**
 * Looks up the value associated with <code>key</code>.
 *
 * @param key The key string to look up the value for.
 * @param def The default value to return if no entry is found.
 * @returns The corresponding value string, or the default string if not found.
 */
string ResourceFile::getValue (const string& key, const string& def) {
  ResourceMap::iterator iter;
  iter = m_entries.find (key);
  if (iter == m_entries.end ()) return def;
  else return iter->second;
}


