/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "SSLClientSocket.hh"

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <fcntl.h>
#include <openssl/ssl.h>
#include <errno.h>
#include <unistd.h>
#include <new>
#include <stdexcept>

using std::bad_alloc;
using std::invalid_argument;
using std::runtime_error;
using std::string;

/**
 * Initializes the SSLClientSocket object.
 *
 * A connection attempt is started and will normally complete (or fail)
 * some unknown time later. The FDHolder object associated with this socket
 * can be used to determine whether a connection has been established or not.
 *
 * @param host The target host to connect to (IP address not accepted).
 * @param port The port number to connect to (in host by ordering).
 * @exception invalid_argument If the hostname is invalid.
 * @exception runtime_error If the connect() call fails immediately.
 */
SSLClientSocket::SSLClientSocket (const string& host, short port)
  : m_fd (socket (AF_INET, SOCK_STREAM, 0)),
    m_fdh (m_fd),
    m_ctx (SSL_CTX_new (SSLv23_client_method ())),
    m_ssl (SSL_new (m_ctx)),
    m_readEOF (false),
    m_writeEOF (false)
{
  if ((m_fd == -1) || (!m_ctx) || (!m_ssl)) throw bad_alloc ();

  if (fcntl (m_fd, F_SETFL, O_NONBLOCK) == -1) throw bad_alloc ();

  if (!SSL_set_fd (m_ssl, m_fd)) throw bad_alloc ();

  hostent *he = gethostbyname (host.c_str ());
  if (!he) throw invalid_argument ("hostname not found: " + host);
  if (he->h_addrtype != AF_INET)
    throw invalid_argument ("unknown address type for: " + host);

  sockaddr_in sin;
  bzero (&sin, sizeof (sin));
#if defined(__FreeBSD__) || defined(__NetBSD)
  sin.sin_len = sizeof (sin);
#endif
  sin.sin_family = AF_INET;
  sin.sin_port = htons (static_cast<short> (port));
  sin.sin_addr.s_addr = *(in_addr_t *)(he->h_addr_list[0]);

  if (connect (m_fd, (sockaddr *)&sin, sizeof (sin)) == -1) {
    if (errno != EINPROGRESS) { // only EINPROGRESS is an okay condition
      close (m_fd);
      throw runtime_error ("unable to connect to: " + host);
    }
    m_fdh.setWriteWanted (true); // waiting for connect to complete
  }

  SSL_set_connect_state (m_ssl); // We want transparent handshake
}

/**
 * Destroys the socket.
 *
 * Attempts to do a proper SSL shutdown before closing down the underlying
 * socket.
 */
SSLClientSocket::~SSLClientSocket () {
  if (m_ssl) SSL_shutdown (m_ssl);
  if (m_fd != -1) close (m_fd);
}


/**
 * Returns the FDHolder used by this object.
 *
 * @returns The FDHolder used by this object.
 */
FDHolder& SSLClientSocket::getFDHolder () {
  return m_fdh;
}

/**
 * Reads data from the SSL link
 *
 * @param data The string object the data will be </i>appended</i> to.
 * @returns > 0 if data is read, 0 if no data available (in which case the
 *     FDHolder contains the read/write status), and -1 on error/closed link.
 */
int SSLClientSocket::read (string& data) {
  if (!m_fdh.canRead ()) {
    m_fdh.setReadWanted (true);
    return 0;
  }

  m_fdh.setReadWanted (false); // Clear outstanding read request, if any

  char buffer[512];
  int ret = SSL_read (m_ssl, buffer, sizeof (buffer));
  if (ret > 0) {
    data.append (buffer, ret);
    return ret;
  }
  else {
    int err = SSL_get_error (m_ssl, ret);

    if (err == SSL_ERROR_WANT_READ) m_fdh.setReadWanted (true);
    else if (err == SSL_ERROR_WANT_WRITE) m_fdh.setWriteWanted (true);
    else if (err == SSL_ERROR_ZERO_RETURN) {
      m_readEOF = true;
      return -1;
    }
    else return -1;
  }

  return 0;
}

/**
 * Writes the data to the SSL link.
 *
 * @param data The object containing the data to be written.
 * @returns > 0 if data was successfully written, 0 if the request could not
 *     be satisfied at this time, and -1 if an error/link closure has occurred.
 */
int SSLClientSocket::write (string& data) {
  if (!m_fdh.canWrite ()) {
    m_fdh.setWriteWanted (true);
    return 0;
  }

  m_fdh.setWriteWanted (false); // Clear outstanding write request, if any

  int ret = SSL_write (m_ssl, data.data (), data.length ());
  if (ret > 0) return ret;
  else {
    int err = SSL_get_error (m_ssl, ret);

    if (err == SSL_ERROR_WANT_READ) m_fdh.setReadWanted (true);
    else if (err == SSL_ERROR_WANT_WRITE) m_fdh.setWriteWanted (true);
    else if (err == SSL_ERROR_ZERO_RETURN) {
      m_writeEOF = true;
      return -1;
    }
    else return -1;
  }

  return 0;
}


/**
 * Checks whether an EOF has been encountered on the input stream.
 *
 * @returns True if an EOF has been encountered on the input stream, false
 *     otherwise.
 */
bool SSLClientSocket::isReadEOF () {
  return m_readEOF;
}

/**
 * Checks whether an EOF has been encountered on the output stream.
 *
 * @returns True if an EOF has been encountered on the output stream, false
 *     otherwise.
 */
bool SSLClientSocket::isWriteEOF () {
  return m_writeEOF;
}
