/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "SampleTransformer.hh"

#include <algorithm>

using std::max;

/**
 * Creates an empty sample vector with a normalization max of 100, and a
 * normalization percentage of 98%.
 */
SampleTransformer::SampleTransformer ()
  : m_includeSize (0), m_outputVectorSize (0), m_normalizationMax (100),
    m_normalizationPercentage (98)
{
  // Empty body
}

/**
 * Destroys the SampleTransformer.
 */
SampleTransformer::~SampleTransformer () {
  // Empty body
}

/**
 * Assign a new input sample vector.
 *
 * A new output vector is calculated based on the current settings of the
 * SampleTransformer. The original unmodified data is also kept (copied)
 * for future use should any other setting change that requires the output
 * sample vector to be recalculated.
 *
 * @param newData The new sample vector to use as input.
 */
void SampleTransformer::setSampleVector (const SampleVector& newData) {
  m_inputSamples = newData;

  updateOutputVector ();
}

/**
 * Sets the number of input samples to include in the calculation of the
 * output vector.
 *
 * The last <code>size</code> samples in the input sample vector will be
 * used when computing the output vector. The output vector is recomputed
 * with the new include size.
 *
 * @param size The number of samples to include.
 */
void SampleTransformer::setIncludeSize (int size) {
  m_includeSize = size;

  updateOutputVector ();
}

/**
 * Sets the size of the generated output vector.
 *
 * The generated sample vector will contain <code>size</code> elements.
 * The output vector is recalculated with the new output size.
 *
 * @param size The new number of elements the generated output vector should
 *     have.
 */
void SampleTransformer::setOutputVectorSize (int size) {
  m_outputVectorSize = size;

  updateOutputVector ();
}

/**
 * Sets the normalization level to be used.
 *
 * The normalization level determines what the maximum allowed value in the
 * output sample vector is, and the maximum percentage indicates how much of
 * that max value the highest found peak should have.
 *
 * Default values are 100, and 98%.
 *
 * @param maxValue The maximum allowed value in the output vector.
 * @param maxPercentage The percentage of <code>maxValue</code> that the highest
 *     found peak should have. Values over 100% are non-sensical and results of
 *     using such a value are undefined.
 */
void SampleTransformer::setNormalizationLevel (int maxValue, int maxPercentage) {
  m_normalizationMax = maxValue;
  m_normalizationPercentage = maxPercentage;

  updateOutputVector ();
}

/**
 * Returns the peak value found in the input sample vector.
 *
 * @returns The peak value found in the input sample vector.
 */
double SampleTransformer::getPeakValue () const {
  SampleVector temp;
  createAdjustedInputData (temp);

  if (!temp.size ()) return 0.0;

  double peak = *max_element (temp.begin (), temp.end ());
  return peak;
}

/**
 * Returns the generated output vector.
 *
 * The generated output vector will have been normalized to the specifications
 * given to this object.
 *
 * @returns The generated sample output vector.
 */
const SampleVector& SampleTransformer::getOutputVector () const {
  return m_outputSamples;
}

/**
 * Recalculates the sample output vector.
 */
void SampleTransformer::updateOutputVector () {
  m_outputSamples.clear ();

  if (!m_outputVectorSize) return;
  if (!m_includeSize || !m_inputSamples.size ()) {
    // No input data, fill output with zeroes
    m_outputSamples.assign (m_outputVectorSize, 0);
    return;
  }

  SampleVector adjustedInput;
  createAdjustedInputData (adjustedInput);
  // Now we know we have m_includeSize elements available for use

  double step = static_cast<double> (m_includeSize) / m_outputVectorSize;

  double sourceIndex = 0;
  double lastSourceIndex = 0;

  for (int i = 0; i < m_outputVectorSize; ++i) {
    int currentIndex = static_cast<int> (sourceIndex);
    int lastIndex = static_cast<int> (lastSourceIndex);
    int numSamples = currentIndex - lastIndex;

    double value = adjustedInput[currentIndex];

    // Find peak between last used index and current
    for (int i = 0; i < numSamples; ++i) {
      value = max (value, adjustedInput[lastIndex +i]);
    }

    m_outputSamples.push_back (value);

    lastSourceIndex = sourceIndex;
    sourceIndex += step;
  }

  // Last but not least, normalize so we keep all the values
  // within the desired range
  normalizeOutputVector ();
}

/**
 * Helper function to ensure that the input data is shrunk or expanded to the
 * right size.
 *
 * @param resultData The resulting input sample vector.
 */
void SampleTransformer::createAdjustedInputData (SampleVector& resultData) const
{
  resultData.clear ();

  SampleVector temp;
  temp.assign (m_includeSize, 0);
  copy (m_inputSamples.begin (), m_inputSamples.end (), back_inserter (temp));

  int startAt = temp.size () - m_includeSize;
  SampleVector::iterator iter = temp.begin ();
  advance (iter, startAt);

  resultData.assign (iter, temp.end ());
}

/**
 * Normalizes the output vector to the given specifications.
 */
void SampleTransformer::normalizeOutputVector () {
  if (!m_outputSamples.size ()) return;

  double maxSample = *max_element (m_outputSamples.begin (),
      m_outputSamples.end ());

  double maxDesiredResult = m_normalizationMax * m_normalizationPercentage;
  maxDesiredResult /= 100;

  double scaleFactor = maxDesiredResult / maxSample;

  SampleVector::iterator iter = m_outputSamples.begin ();
  SampleVector::iterator end = m_outputSamples.end ();

  for (; iter != end; ++iter) *iter *= scaleFactor;
}


