/* Copyright (C) 2004, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "SimpleCalendar.hh"

#include <cstring>
#include <stdexcept>

using std::invalid_argument;
using std::runtime_error;


/**
 * Initialize based on a date in ISO format (YYYYMMDD).
 *
 * @param isoStartDate The initial date for the calendar, in YYYYMMDD format.
 * @throws invalid_argument If mktime cannot make sense out of the given date.
 */
SimpleCalendar::SimpleCalendar (unsigned long isoStartDate) {
  struct tm startDate;
  bzero (&startDate, sizeof (startDate));

  int year = isoStartDate / 10000; // Extract the year part
  startDate.tm_year = year - 1900; // Offset from year 1900
  isoStartDate -= year * 10000; // Get rid of the year for easier handling

  int month = isoStartDate / 100; // Extract the month part
  startDate.tm_mon = month -1; // Zero-based month counting
  isoStartDate -= month * 100; // Remove the month

  startDate.tm_mday = isoStartDate;

  m_time = mktime (&startDate);
  if (-1 == m_time) throw invalid_argument ("date is not representable!");
}

/**
 * Simple destructor.
 */
SimpleCalendar::~SimpleCalendar () {
}


/**
 * Return the date the calendar is currently at.
 *
 * @returns The date in ISO format (YYYYMMDD).
 */
unsigned long SimpleCalendar::isoDate () const {
  struct tm *date;

  date = localtime (&m_time);

  unsigned long isoDate (0);
  isoDate = (date->tm_year + 1900) * 10000;
  isoDate += (date->tm_mon +1) * 100;
  isoDate += date->tm_mday;

  return isoDate;
}


/**
 * Advance the calendar <code>numDays</code> days.
 *
 * @param numDays The number of days to advance the calendar by.
 * @throws runtime_exception If the date is not representable after the
 *                           avancement.
 */
void SimpleCalendar::advanceDays (int numDays) {
  struct tm *date;

  date = localtime (&m_time);
  date->tm_mday += numDays;

  m_time = mktime (date);
  if (-1 == m_time)
      throw runtime_error ("date not representable after advancement");
}
