/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "SpeedDataHandler.hh"
#include "PanelHandler.hh"
#include "ERegEx.hh"
#include <sstream>
#include <iostream>

using std::cerr;
using std::endl;
using std::istringstream;
using std::string;

/**
 * Initializes the object.
 *
 * @param panel The PanelHandler to use for communicating information to the
 *     status panel.
 * @see PanelHandler
 */
SpeedDataHandler::SpeedDataHandler (PanelHandler& panel)
  : m_panel (panel)
{
  // Empty body
}

/**
 * Destroys the object.
 */
SpeedDataHandler::~SpeedDataHandler () {
  // Empty body
}

/**
 * Interprets the HTTP response (including header).
 *
 * Attempts to validate the response as far as possible, and may print
 * diagnostics to <code>cerr</code>. If a format error is discovered in the
 * actual speed data, the status panel's text is also updated to notify the
 * user that something is really amiss.
 *
 * @param data The full HTTP response string to be processed.
 */
void SpeedDataHandler::processData (string& data) {
  istringstream extractor (data);
  string resultCode;
  if (!getline (extractor, resultCode)) {
    cerr << "Error: speedcheck: HTTPS server returned malformed header!\n";
    return;
  }

  if (resultCode.find ("200") == string::npos) {
    // Failed HTTP method
    cerr << "Error: speedcheck: HTTPS server reports failure: ";
    cerr  << resultCode << endl;
    return;
  }

  string line;
  while (getline (extractor, line)) {
    if (!line.length ()) break; // Respond to LF as blank line
    if (line.length () == 1) break; // Also allow CRLF as blank line
  }

  if (!getline (extractor, line)) { // No body??
    cerr << "Error: speedcheck: HTTPS server returned no data!\n";
    return; 
  }

  StatusReceiver& statusReceiver = m_panel.getStatusReceiver ();

  // Check so the data is in the right format
  ERegEx expectedFormat ("((SDSL )|())[0-9]+((\\.[0-9]+)|()) [KM]bits/sec");
  if (!expectedFormat.isMatch (line)) {
    cerr << "Error: Unrecognized format of speed data: " << line << endl;
    statusReceiver.setErrorText (" ERROR! ");
    m_panel.reDraw ();
    return;
  }

  if (line.find ("SDSL") != string::npos) {
    statusReceiver.setDslText ("SDSL");
    line = line.substr (5, string::npos); // Skip the SDSL part
  }
  else {
    statusReceiver.setDslText ("ADSL");
  }

  extractor.str (line);
  double speed = 0;
  extractor >> speed;
  if (speed < 2) speed = 1500; // turn 1.5 into 1500 instead

  statusReceiver.setSpeed (static_cast<int> (speed));

  m_panel.reDraw ();
}
