/* Copyright (C) 2002-2005, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "StatsFetcher.hh"

#include "FDSet.hh"
#include "DataHandler.hh"
#include "Request.hh"
#include "SSLClientSocket.hh"
#include "version.h"
#include <stdexcept>
#include <sstream>
#include <iostream>

using std::cerr;
using std::endl;
using std::invalid_argument;
using std::ostringstream;
using std::runtime_error;
using std::string;

/**
 * Initializes the StatsFetcher object.
 *
 * @param fdset The FDSet to add any requests' FDHolders to.
 * @param username The username to use when building the request strings.
 * @param password The password to use when building the request strings.
 * @param host The hostname for the requests to connect to.
 * @param url The URL of the usage CGI script.
 * @param usage The DataHandler to be handed the response to any usage queries.
 * @param speed The DataHandler to be handed the response to any speed queries.
 * @param history The DataHandler to be handed the response to any history
 *     queries.
 * @param snoop Set to true to dump data to stderr before processing.
 * @see FDHolder
 * @see DataHandler
 */
StatsFetcher::StatsFetcher (
    FDSet& fdset,
    const string& username,
    const string& password,
    const string& host,
    const string& url,
    DataHandler& usage,
    DataHandler& speed,
    DataHandler& history,
    bool snoop)
  : m_fdSet (fdset), m_username (username), m_password (password),
    m_host (host), m_url (url), m_usageHandler (usage), m_speedHandler (speed),
    m_historyHandler (history), m_usageRequest (0), m_speedRequest (0),
    m_historyRequest (0), m_snoop (snoop)
{
  // Empty body
}

/**
 * Destroys the object and any outstanding requests.
 */
StatsFetcher::~StatsFetcher () {
  clearUsageRequest ();
  clearSpeedRequest ();
  clearHistoryRequest ();
}

/**
 * Queues a new usage check request.
 *
 * If there is already an outstanding usage request, this old request is
 * destroyed and replaced with a new one (to ensure an old hung request
 * doesn't block future checks).
 *
 * @param update Controls whether fresh data should be requested from the
 *     billing system. This must only be set to True in response to direct
 *     user action. Should normally be false.
 */
void StatsFetcher::queueUsageCheck (bool update) {
  // discard any old request in progress
  if (m_usageRequest) clearUsageRequest ();

  SSLClientSocket* sock = createSSLSocket ();
  if (!sock) return;

  string data = buildBasicDataString ();
  if (update) data += "update=1&";
  string request = buildRequestString (data);

  try {
    m_usageRequest = new Request (sock, request, m_usageHandler, m_snoop);
    m_fdSet.addFDHolder (m_usageRequest->getFDHolder ());
  }
  catch (invalid_argument& ia) {
    cerr << "Internal error: " << ia.what () << endl;
  }
}

/** 
 * Queues a new speed check request.
 *
 * If there is already an outstanding speed request, this old request is 
 * destroyed and replaced with a new one (to ensure an old hung request 
 * doesn't block future checks).
 *
 * @param update Controls whether fresh data should be requested from the
 *     billing system. This must only be set to True in response to direct
 *     user action. Should normally be false.
 */
void StatsFetcher::queueSpeedCheck (bool update) {
  // discard any old request in progress
  if (m_speedRequest) clearSpeedRequest ();

  SSLClientSocket* sock = createSSLSocket ();
  if (!sock) return;

  string data = buildBasicDataString ();
  data += "speed=1&";
  if (update) data += "update=1&";
  string request = buildRequestString (data);

  try {
    m_speedRequest = new Request (sock, request, m_speedHandler, m_snoop);
    m_fdSet.addFDHolder (m_speedRequest->getFDHolder ());
  }
  catch (invalid_argument& ia) {
    cerr << "Internal error: " << ia.what () << endl;
  }
}

/** 
 * Queues a new history check request.
 *
 * If there is already an outstanding history request, this old request is 
 * destroyed and replaced with a new one (to ensure an old hung request 
 * doesn't block future checks).
 *
 * @param update Controls whether fresh data should be requested from the
 *     billing system. This must only be set to True in response to direct
 *     user action. Should normally be false.
 */
void StatsFetcher::queueHistoryCheck (bool update) {
  // discard any old request in progress
  if (m_historyRequest) clearHistoryRequest ();

  SSLClientSocket *sock = createSSLSocket ();
  if (!sock) return;

  string data = buildBasicDataString ();
  data += "history=1&";
  if (update) data += "update=1&";
  string request = buildRequestString (data);

  try {
    m_historyRequest = new Request (sock, request, m_historyHandler, m_snoop);
    m_fdSet.addFDHolder (m_historyRequest->getFDHolder ());
  }
  catch (invalid_argument& ia) {
    cerr << "Internal error: " << ia.what () << endl;
  }
}

/**
 * Walks the list of outstanding requests and checks whether any of them
 * has a network event that should be responded to.
 *
 * If an event is found, it is possible that a request will have completed
 * after that event has been handled, and if so, it will be cleared (destroyed).
 */
void StatsFetcher::processNetworkEvent () {
  if (m_usageRequest && m_usageRequest->getFDHolder ().hasEvent () &&
      m_usageRequest->processNetworkEvent ()) { // Completed
    clearUsageRequest ();
  }
  if (m_speedRequest && m_speedRequest->getFDHolder ().hasEvent () &&
      m_speedRequest->processNetworkEvent ()) { // Completed
    clearSpeedRequest ();
  }
  if (m_historyRequest && m_historyRequest->getFDHolder ().hasEvent () &&
      m_historyRequest->processNetworkEvent ()) { // Completed
    clearHistoryRequest ();
  }
}

/**
 * Destroys an outstanding usage request, if any.
 */
void StatsFetcher::clearUsageRequest () {
  if (m_usageRequest) {
    m_fdSet.removeFDHolder (m_usageRequest->getFDHolder ());
    delete m_usageRequest;
    m_usageRequest = 0;
  }
}

/**
 * Destroys an outstanding speed request, if any.
 */
void StatsFetcher::clearSpeedRequest () {
  if (m_speedRequest) {
    m_fdSet.removeFDHolder (m_speedRequest->getFDHolder ());
    delete m_speedRequest;
    m_speedRequest = 0;
  }
}

/**
 * Destorys an outstanding history request, if any.
 */
void StatsFetcher::clearHistoryRequest () {
  if (m_historyRequest) {
    m_fdSet.removeFDHolder (m_historyRequest->getFDHolder ());
    delete m_historyRequest;
    m_historyRequest = 0;
  }
}

/**
 * Convenience function to create a new SSL socket.
 *
 * Catches exceptions thrown at socket creation time and prints the appropriate
 * diagnostics on <code>cerr</code>.
 *
 * @returns The new SSLClientSocket, or null if creation failed.
 */
SSLClientSocket *StatsFetcher::createSSLSocket () {
  SSLClientSocket *sock = 0;
  try {
    sock = new SSLClientSocket (m_host, 443);
  }
  catch (runtime_error& re) {
    cerr << re.what () << endl;
  }
  catch (invalid_argument& ia) {
    cerr << "Internal error: " << ia.what () << endl;
  }

  return sock;
}

/**
 * Assembles the basic HTTP data string used in all requests.
 *
 * @returns The created string.
 */
string StatsFetcher::buildBasicDataString () {
  // Should possibly encode the @ in the username...
  string data = "username=" + m_username + "&";
  data += "password=" + m_password + "&";
  data += "iso=1&";

  return data;
}

/**
 * Creates the final HTTP request string.
 *
 * @param data The HTTP data that should be included in the HTTP request.
 * @returns The complete HTTP request string.
 */
string StatsFetcher::buildRequestString (const string& data) {
  ostringstream request;
  request << "POST " << m_url << " HTTP/1.0\r\n";
  request << "User-Agent: " << APPNAME << "/" << VERSION << "\r\n";
  request << "Connection: close\r\n";
  request << "Content-type: text/plain\r\n";
//  request << "Content-encoding: application/x-www-form-urlencoded\r\n";
  request << "Content-length: " << data.length () << "\r\n";

  request << "\r\n";
  request << data;

  return request.str ();
}

