/* Copyright (C) 2002-2004, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "StatusPainter.hh"
#include "xpm/statusmask.xbm"
#include "xpm/field_s.xpm"
#include "xpm/field_l.xpm"
#include <sstream>
#include <iomanip>
#include <algorithm>

using std::ostringstream;
using std::setfill;
using std::setw;
using std::string;
using std::swap;

/**
 * Initializes the StatusPainter object and ties it to a Display and
 * WMDockWindow.
 *
 * @param dpy The X Display to tie this object to.
 * @param window The WMDockWindow to use for output.
 */
StatusPainter::StatusPainter (Display& dpy, WMDockWindow& window)
  : m_display (dpy), m_window (window), m_charset (dpy),
    m_dsl (dpy, window, m_charset, 4, 4, field_s_xpm),
    m_speed (dpy, window, m_charset, 33, 4, field_s_xpm),
    m_meter (dpy, window, 4, 18),
    m_usage (dpy, window, m_charset, 4, 34, field_l_xpm),
    m_rollOver (dpy, window, m_charset, 4, 47, field_l_xpm),
    m_max (0), m_cur (0), m_alternateDateFormat (false)
{
  m_mask = XCreateBitmapFromData (&dpy, XDefaultRootWindow (&dpy),
      reinterpret_cast<const char*> (statusmask_bits), statusmask_width,
      statusmask_height);

  m_dsl.setText (" DSL");
  m_speed.setText ("    ");
  m_usage.setText ("QUERYING ");
  m_rollOver.setText ("R/O:     ");
}

/**
 * Destroys this object and frees associated resources.
 */
StatusPainter::~StatusPainter () {
  XFreePixmap (&m_display, m_mask);
}

/**
 * Updates the text to be displayed in the "DSL" field.
 *
 * Does not cause an XFlush.
 *
 * @param text The new text to be displayed.
 */
void StatusPainter::setDslText (const string& text) {
  m_dsl.setText (text);
}

/**
 * Updates the text to be displayed in the speed field.
 *
 * This is to be used for signalling severe errors to the user.
 * Does not cause an XFlush.
 *
 * @param text The new text to be displayed.
 */
void StatusPainter::setErrorText (const string& text) {
  m_usage.setText (text);
}

/**
 * Updates the speed string displayed.
 *
 * Does not cause an XFlush.
 *
 * @param speed The new speed value.
 */
void StatusPainter::setSpeed (int speed) {
  ostringstream str;
  str << setw (4) << setfill (' ') << speed;
  m_speed.setText (str.str ());
}

/**
 * Updates the max value for the usage field.
 *
 * The usage field is recomputed with the new max value in place, and the
 * usage meter is updated to match.
 *
 * Does not cause an XFlush.
 *
 * @param max The new maximum value for the usage field.
 */
void StatusPainter::setMaxLimit (int max) {
  m_meter.setMaxValue (max);
  m_max = max;

  updateUsageString ();
}

/**
 * Updates the current usage value for the usage field, if not on flat rate.
 *
 * The usage field is recomputed with the new usage value in place, and the
 * usage meter is updated to match.
 *
 * Note that flat rate is determined by whether the max limit is zero or not.
 *
 * Does not cause an XFlush.
 *
 * @param used The new usage value.
 */
void StatusPainter::setUsage (int used) {
  if (m_max == 0) return; // flat rate, use setRollingUsage instead

  m_meter.setCurrentValue (used);
  m_cur = used;

  updateUsageString ();
}

/**
 * Updates the usage value for the usage field, if on flat rate plan.
 *
 * The usage field is recomputed with the new usage value in place.
 *
 * Note that flat rate is determined by whether the max limit it zero or not.
 *
 * Does not cause an XFlush
 *
 * @param used The new usage value.
 */
void StatusPainter::setRollingUsage (int used) {
  if (m_max > 0) return; // not flat rate, use setUsage instead

  m_cur = used;

  updateUsageString ();
}

/**
 * Updates the rollover date field.
 *
 * Depending on whether alternate date format is used or not the new text is
 * either in month/day format of day/month format (default).
 *
 * Does not cause an XFlush.
 *
 * @param day The new rollover day, as an integer [1..~31], no checking done.
 * @param month The new rollover month, as an integer [1..12], no checking done.
 */
void StatusPainter::setRollOverDate (int day, int month) {
  if (m_alternateDateFormat) swap (day, month);

  ostringstream str;
  str << "R/O:" << setw (2) << setfill ('0') << day;
  str << "/" << setw (2) << setfill ('0') << month;
  m_rollOver.setText (str.str ());
}

/**
 * Reapplies the shape mask to the WMDockWindow.
 */
void StatusPainter::resetMask () {
  m_window.setMask (m_mask);
}

/**
 * Redraws the status panel and issues an XFlush.
 */
void StatusPainter::reDraw () {
  m_dsl.reDraw ();
  m_speed.reDraw ();
  m_meter.reDraw ();
  m_usage.reDraw ();
  m_rollOver.reDraw ();

  XFlush (&m_display);
}

/**
 * Toggles the use of alternate date format.
 *
 * Default is to use day/month format.
 */
void StatusPainter::toggleDateFormat () {
  m_alternateDateFormat = !m_alternateDateFormat;
}

/**
 * Helper function to generate the usage string based on the maximum and used
 * values.
 */
void StatusPainter::updateUsageString () {
  ostringstream str;
  bool flatRate (m_max == 0);

  if (flatRate) {
    str << "FR:" << setw (6) << setfill (' ') << m_cur;
  }
  else {
    if (m_max > 1000) { // use new format (nnnnn/nnG)
      str << setw (5) << setfill (' ') << m_cur;
      str << "/" << setw (2) << setfill (' ');
      str << (m_max / 1000) << "G";
    }
    else { // use old format (nnnn/nnnn)
      str << setw (4) << setfill (' ') << m_cur;
      str << "/" << setw (4) << setfill (' ') << m_max;
    }
  }

  m_usage.setText (str.str ());
}

