/* Copyright (C) 2002-2003, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "UsageDataHandler.hh"
#include "PanelHandler.hh"
#include "ERegEx.hh"

#include <sstream>
#include <iostream>

using std::cerr;
using std::endl;
using std::istringstream;
using std::string;

/**
 * Initializes the object.
 *
 * @param panel The PanelHandler to use for communicating information to the
 *     status panel.
 * @see PanelHandler
 */
UsageDataHandler::UsageDataHandler (PanelHandler& panel)
  : m_panel (panel)
{
  // Empty body
}

/**
 * Destroys the object
 */
UsageDataHandler::~UsageDataHandler () {
  // Empty body
}

/**
 * Interprets the HTTP response (including header).
 *
 * Attempts to validate the response as far as possible, and may print
 * diagnostics to <code>cerr</code>. If a format error is discovered in the
 * actual speed data, the status panel's text is also updated to notify the
 * user that something is really amiss.
 *
 * @param data The full HTTP response string to be processed.
 */
void UsageDataHandler::processData (string& data) {
  StatusReceiver& statusReceiver = m_panel.getStatusReceiver ();

  istringstream extractor (data);
  string resultCode;
  if (!getline (extractor, resultCode)) {
    cerr << "Error: usagecheck: HTTPS server returned malformed header!\n";
    statusReceiver.setErrorText (" ERROR! ");
    m_panel.reDraw ();
    return;
  }

  if (resultCode.find ("200") == string::npos) {
    // Failed HTTP method
    cerr << "Error: usagecheck: HTTPS server reports failure: ";
    cerr << resultCode << endl;
    statusReceiver.setErrorText (" ERROR! ");
    m_panel.reDraw ();
    return;
  }

  string line;
  while (getline (extractor, line)) {
    if (!line.length ()) break; // Respond to LF as blank line
    if (line.length () == 1) break; // Also allow CRLF as blank line
  }

  if (!getline (extractor, line)) { // No body??
    cerr << "Error: usagecheck: HTTPS server returned no data!\n";
    statusReceiver.setErrorText (" ERROR! ");
    m_panel.reDraw ();
    return; 
  }

  // Ensure we got it in the right format, or abort before we start
  // working on the data
  ERegEx expectedFormat ("-?[0-9]+(\\.[0-9]+)? [0-9]+ [0-9]{8} [0-9]+(\\.[0-9]+)?");
  if (!expectedFormat.isMatch (line)) {
    cerr << "Error: Unrecognized format of usage data: " << line << endl;
    statusReceiver.setErrorText (" ERROR! ");
    m_panel.reDraw ();
    return;
  }

  extractor.str (line);
  double usedvol;
  extractor >> usedvol;
  int used = static_cast<int> (usedvol);

  int maxvol;
  extractor >> maxvol;
  statusReceiver.setMaxLimit (maxvol); // note: must set this before usage, else
  statusReceiver.setUsage (used);      //  flat rate is assumed by StatusPainter

  string date;
  extractor >> date;
  extractor.str (date.substr (4, 2));
  int month;
  extractor >> month;
  istringstream extractor2 (date.substr (6, 2));
  int day;
  extractor2 >> day;
  statusReceiver.setRollOverDate (day, month);

  m_panel.reDraw ();
}
