/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "WMDockWindow.hh"
#include "XResource.hh"

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/xpm.h>
#include <X11/extensions/shape.h>

#include <new>

using std::bad_alloc;
using std::string;

/**
 * Creates a window suitable for use on a WMDock.
 *
 * @param dpy The X Display to create the window on.
 * @param name The name to use as resource class and name.
 * @param size The size of the window (square window).
 * @exception bad_alloc If something cannot be allocated.
 */
WMDockWindow::WMDockWindow (Display& dpy, const string& name, int size)
  : m_display (dpy), m_gc (0)
{
  XSizeHints *sizeHints = XAllocSizeHints ();
  XResource sizeHintsRes (sizeHints);

  sizeHints->flags = PMinSize | PMaxSize | PSize | PWinGravity;
  sizeHints->min_width = sizeHints->min_height = size;
  sizeHints->max_width = sizeHints->max_height = size;
  sizeHints->base_width = sizeHints->base_height = size;
  sizeHints->win_gravity = NorthWestGravity;

  int screen = XDefaultScreen (&dpy);
  Pixel blackPixel = XBlackPixel (&dpy, screen);
  Window rootWindow = XDefaultRootWindow (&dpy);

  m_window = XCreateSimpleWindow (&dpy, rootWindow, 0, 0, sizeHints->base_width,
      sizeHints->base_height, 0, blackPixel, blackPixel);
  m_iconWindow = XCreateSimpleWindow (&dpy, m_window, 0, 0,
      sizeHints->base_width, sizeHints->base_height, 0, blackPixel, blackPixel);

  XSetWMNormalHints (&dpy, m_window, sizeHints);

  long eventMask = ButtonPressMask | ButtonReleaseMask | ExposureMask | \
      StructureNotifyMask;
  XSelectInput (&dpy, m_window, eventMask);
  XSelectInput (&dpy, m_iconWindow, eventMask);

  // ugly to use casts below
  XClassHint *classHint = XAllocClassHint ();
  XResource classHintRes (classHint);
  classHint->res_name = const_cast<char *> (name.c_str ());
  classHint->res_class = const_cast<char *> (name.c_str ());
  XSetClassHint (&dpy, m_window, classHint);

  XWMHints *wmHints = XAllocWMHints ();
  XResource wmHintsRes (wmHints);
  wmHints->initial_state = WithdrawnState;
  wmHints->icon_window = m_iconWindow;
  wmHints->window_group = m_window;
  wmHints->flags = StateHint | IconWindowHint | WindowGroupHint;
  XSetWMHints (&dpy, m_window, wmHints);

  XTextProperty wmName;
  if (!XStringListToTextProperty (&classHint->res_class, 1, &wmName)) {
    throw bad_alloc (); // careful not to leak anything XAlloc'd here
  }
  XSetWMName (&dpy, m_window, &wmName);
  XFree (wmName.value);

  XGCValues gcValues;
  gcValues.graphics_exposures = 0;
  m_gc = XCreateGC (&dpy, rootWindow, GCGraphicsExposures, &gcValues);
}

/**
 * Destroys the window and releases resources used.
 */
WMDockWindow::~WMDockWindow () {
  XDestroyWindow (&m_display, m_iconWindow);
  XDestroyWindow (&m_display, m_window);
  XFreeGC (&m_display, m_gc);
}

/**
 * Applies the given shape mask.
 *
 * @param mask The Pixmap to use as the new shape mask.
 */
void WMDockWindow::setMask (Pixmap& mask) {
  XShapeCombineMask (&m_display, m_window, ShapeBounding, 0, 0, mask, ShapeSet);
  XShapeCombineMask (&m_display, m_iconWindow, ShapeBounding, 0, 0, mask,
      ShapeSet);
}

/**
 * Determines whether the window wil be mapped onto the display or not.
 *
 * @param visible True if the window should be mapped, false if not.
 */
void WMDockWindow::setVisible (bool visible) {
  if (visible) XMapWindow (&m_display, m_window);
  else XUnmapWindow (&m_display, m_window);
}

/**
 * Draws the given Pixmap onto the window at the specified coordinates.
 *
 * @param xpm The Pixmap to draw from.
 * @param x The X coordinate into the <code>xpm</code>.
 * @param y The Y coordinate into the <code>xpm</code>.
 * @param cx The number of pixel columns of <code>xpm</code> to draw.
 * @param cy The number of pixel rows of <code>xpm</code> to draw.
 * @param atx The X coordinate of the window where to draw the Pixmap.
 * @param aty The Y coordinate of the window where to draw the Pixmap.
 */
void WMDockWindow::drawXpmAt (const Pixmap& xpm, int x, int y, int cx, int cy,
  int atx, int aty) {
  XCopyArea (&m_display, xpm, m_window, m_gc, x, y, cx, cy, atx, aty);
  XCopyArea (&m_display, xpm, m_iconWindow, m_gc, x, y, cx, cy, atx, aty);
}

