/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "XpmGraphField.hh"
#include "xpm/graphbar.xpm"
#include <new>

using std::bad_alloc;

/**
 * Initializes the object and ties it to the specified Display.
 *
 * @param dpy The X Display to tie this object to.
 * @param window The WMDockWindow this field should appear on.
 * @param x The X coordinate on the WMDockWindow where the field should be.
 * @param y The Y coordinate on the WMDockWindow where the field should be.
 * @param xpmData The XPM data to use as field background. The field size
 *     information is extracted from this data.
 * @exception bad_alloc If something cannot be allocated.
 */
XpmGraphField::XpmGraphField (
    Display& dpy,
    WMDockWindow& window,
    int x,
    int y,
    char **xpmData)
  : m_display (dpy), m_window (window), m_x (x), m_y (y), m_frameX (0),
    m_frameY (0)
{
  XpmAttributes attribs;
  attribs.valuemask = XpmSize;

  if (XpmCreatePixmapFromData (&dpy, XDefaultRootWindow (&dpy), xpmData,
      &m_frame, 0, &attribs) != XpmSuccess) {
    throw bad_alloc ();
  }
  if (XpmCreatePixmapFromData (&dpy, XDefaultRootWindow (&dpy), graphbar_xpm,
      &m_bar, 0, 0) != XpmSuccess) {
    throw bad_alloc ();
  }

  m_frameX = attribs.width;
  m_frameY = attribs.height;
}

/**
 * Destroys the object and frees used resources.
 */
XpmGraphField::~XpmGraphField () {
  XFreePixmap (&m_display, m_frame);
  XFreePixmap (&m_display, m_bar);
}

/**
 * Sets the graph data to be displayed.
 *
 * This data should be normalized and ready for use.
 *
 * @param data The normalized data to be displayed as a graph.
 */
void XpmGraphField::setGraphData (const SampleVector& data) {
  m_graphData = data;
}

/**
 * Redraws the graph onto the associated window.
 */
void XpmGraphField::reDraw () {
  m_window.drawXpmAt (m_frame, 0, 0, m_frameX, m_frameY, m_x, m_y);

  // superimpose the actual graph
  int x = 1 + m_x;
  int baseline = m_frameY + m_y -2;

  SampleVector::iterator iter = m_graphData.begin ();
  SampleVector::iterator end = m_graphData.end ();
  for (; iter != end; ++iter, ++x) {
    int value = static_cast<int> (*iter);
    int ytop = baseline - value;

    //m_window.drawLine (x, baseline, x, ytop);
    m_window.drawXpmAt (m_bar, 0, 0, 1, value +1, x, ytop);
  }
}

/**
 * Returns the width available for drawing the actual graph.
 *
 * The field contains borders that the graph may not draw over.
 *
 * @returns The pixel width of the actual graph drawing area.
 */
int XpmGraphField::getGraphWidth () const {
  return m_frameX -2;
}

/**
 * Returns the height available for drawing the actual graph.
 *
 * The field contains borders that the graph may not draw over.
 *
 * @returns The pixel height of the actual graph drawing area.
 */
int XpmGraphField::getGraphHeight () const {
  return m_frameY -2;
}

