/* Copyright (C) 2002, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "XpmResourceMeter.hh"

#include <new>

#include "xpm/meter_active.xpm"
#include "xpm/meter_inactive.xpm"

using std::bad_alloc;

static const int PIXELSX = 56;
static const int PIXELSY = 14;

/**
 * Initializes the object and ties it to the specified X Display.
 *
 * @param dpy The X Display to tie this object to.
 * @param window The WMDockWindow this meter should appear on.
 * @param x The X coordinate on the WMDockWindow the meter should be at.
 * @param y The Y coordinate on the WMDockWindow the meter should be at.
 * @exception bad_alloc If something cannot be allocated.
 */
XpmResourceMeter::XpmResourceMeter (
    Display& dpy,
    WMDockWindow& window,
    int x,
    int y)
  : m_display (dpy), m_window (window), m_x (x), m_y (y), m_max (100),
    m_cur (0), m_pixelsActive (0)
{
  Window rootWindow = XDefaultRootWindow (&dpy);
  if (XpmCreatePixmapFromData (&dpy, rootWindow, meter_active_xpm, &m_active,
      0, 0) != XpmSuccess) throw bad_alloc ();

  if (XpmCreatePixmapFromData (&dpy, rootWindow, meter_inactive_xpm,
      &m_inactive, 0, 0) != XpmSuccess) {
    XFreePixmap (&dpy, m_active);
    throw bad_alloc ();
  }
}

/**
 * Destroys the object and frees used resources.
 */
XpmResourceMeter::~XpmResourceMeter () {
  XFreePixmap (&m_display, m_active);
  XFreePixmap (&m_display, m_inactive);
}

/**
 * Sets the new maximum value for the resource meter.
 *
 * If the max value is 0, that has the effect of locking the meter at
 * the zero position, irregardless of any value given to setCurrentValue.
 *
 * No recalculation is done. To recalculate the meter's position, issue a
 * setCurrentValue call.
 *
 * @param max The new maxium value.
 */
void XpmResourceMeter::setMaxValue (int max) {
  m_max = max;
  if (!m_max) m_pixelsActive = 0;
}

/**
 * Sets the new current value for the resource meter.
 *
 * The meter's position is recalculated, but no redraw is performed.
 *
 * @param current The new current value.
 */
void XpmResourceMeter::setCurrentValue (int current) {
  m_cur = current;
  if (!m_max) return; // possible divide by zero otherwise...
  double part = static_cast<double> (m_cur) / m_max;
  m_pixelsActive = static_cast<int> (PIXELSX * part);
  if (m_pixelsActive > PIXELSX) m_pixelsActive = PIXELSX;
}

/**
 * Redraws the meter onto the associated window.
 */
void XpmResourceMeter::reDraw () {
  m_window.drawXpmAt (m_active, 0, 0, m_pixelsActive, PIXELSY, m_x, m_y);
  m_window.drawXpmAt (m_inactive, m_pixelsActive, 0, (PIXELSX - m_pixelsActive),
      PIXELSY, m_x + m_pixelsActive, m_y);
}
