/* Copyright (C) 2002-2005, Johny Mattsson
 *
 * This code is licenced under the GNU General Public Licence version 2.
 * See the file COPYING for details.
 */

#include "version.h"
#include "CommandLine.hh"
#include "ResourceFile.hh"
#include "PanelSelector.hh"
#include "FDSet.hh"
#include "StatsFetcher.hh"
#include "UsageDataHandler.hh"
#include "SpeedDataHandler.hh"
#include "HistoryDataHandler.hh"

#include <openssl/ssl.h>
#include <unistd.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <memory>
#include <sstream>
#include <iostream>

using std::auto_ptr;
using std::cerr;
using std::cout;
using std::endl;
using std::istringstream;

// --- Constants
const int ICONSIZE = 64;
const char *const defaultHost = "accounts.internode.on.net";
const char *const defaultURL = "/cgi-bin/padsl-usage";

const char *const user = "username";
const char *const pass = "password";
const char *const host = "host";
const char *const url = "url";

// --- Globals

CommandLineOptions options;
auto_ptr<ResourceFile> rcFile;
bool timeToQuit = false;

Display *dpy;
auto_ptr<WMDockWindow> dockWindow;
auto_ptr<StatsFetcher> fetcher;
FDSet fdSet;

// --- Helper function prototypes
void printBanner ();
void printHelp ();
void waitForMapNotify ();
void scheduleTimer ();

void onTerm (int);
void onAlarm (int);

bool requestUpdatedData ();


// --- Main entry

int main (int argc, char *argv[]) {
  if (!parseCommandLine (argc, argv, options)) {
    cerr << " Try \"" << argv[0] << " -h\" for a list of options\n";
    return 1;
  }

  if (!options.quietMode) printBanner();

  if (options.printHelp) {
    printHelp ();
    return 0;
  }

  rcFile.reset (new ResourceFile (options.rcFile));
  if (!rcFile->load ()) {
    cerr << "Unable to load resource file: " << options.rcFile << endl;
    return 2;
  }
  if (!rcFile->getValue ("username").length ()) {
    cerr << "No \"username\" directive found in " << options.rcFile << "\n";
    return 3;
  }
  if (!rcFile->getValue ("password").length ()) {
    cerr << "No \"password\" directive found in " << options.rcFile << "\n";
    return 4;
  }

  signal (SIGTERM, onTerm);
  signal (SIGINT, onTerm);
  signal (SIGALRM, onAlarm);

  SSL_library_init ();
  SSL_load_error_strings ();

  // PRNG is seeded properly automatically as long as /dev/urandom exists.
  // Thankfully, /dev/urandom does exist on FreeBSD :)
  // If you're compiling this on a system without /dev/urandom, you've got some
  // work ahead of you to seed the PRNG properly ;P

  // At least provide a warning if /dev/urandom appears to be unavailable
  struct stat sb;
  if (stat ("/dev/urandom", &sb) != 0) {
    cerr << "Warning: /dev/urandom not available, security of SSL connection";
    cerr << " may be poor.\n";
  }

  dpy = XOpenDisplay (0);
  if (!dpy) {
    cerr << "Unable to open display. Check your DISPLAY environment variable.";
    cerr << endl;
    return 3;
  }

  dockWindow.reset(new WMDockWindow (*dpy, APPNAME, ICONSIZE));
  dockWindow->setVisible (true);
  waitForMapNotify ();

  PanelSelector panel (*dpy, *dockWindow);
  if (options.alternateDate) panel.getStatusPainter ().toggleDateFormat ();
  switch (options.initialPanel) {
    case 's': panel.selectPanel (STATUSPANEL); break;
    case 'g': panel.selectPanel (HISTORYPANEL); break;
  }
  switch (options.initialGraphMode) {
    case 'w': panel.getHistoryPainter ().setMode (GRAPHWEEK); break;
    case 'm': panel.getHistoryPainter ().setMode (GRAPHMONTH); break;
    case 'q': panel.getHistoryPainter ().setMode (GRAPHQUARTER); break;
    case 'y': panel.getHistoryPainter ().setMode (GRAPHYEAR); break;
  }
  panel.reDraw ();

  // Set up the stats fetcher and wire it all to the panels
  UsageDataHandler usageHandler (panel);
  SpeedDataHandler speedHandler (panel);
  HistoryDataHandler historyHandler (panel);
  fetcher.reset (new StatsFetcher (fdSet, rcFile->getValue (user),
      rcFile->getValue (pass), rcFile->getValue (host, defaultHost),
      rcFile->getValue (url, defaultURL), usageHandler, speedHandler,
      historyHandler, options.snoopData));

  // Queue the initial requests
  fetcher->queueUsageCheck ();
  fetcher->queueSpeedCheck ();
  fetcher->queueHistoryCheck ();

  // Schedule future updates
  scheduleTimer ();

  // Set up the X connection so we are notified when events are available
  FDHolder xfd (XConnectionNumber (dpy));
  xfd.setReadWanted (true);
  fdSet.addFDHolder (xfd);

  // Okay, we're ready do go into our main loop!
  XEvent event;
  while (!timeToQuit) {

    // Wait either for status data to become available, or an X event
    if (!fdSet.checkForEvent (true)) {
      // Wait failed; Unless we restrict ourselves we could easily hog the cpu
      // in case the failure was due to an invalid FD getting stuck in the
      // FDSet. Of course, that will never happen, but just in case... :)
      usleep (100);
    }

    // Deal with the case of only a partial X event being available.
    // That could otherwise lead to another cpu hogging...
    if (xfd.canRead () && !XPending (dpy)) usleep (100);

    // Check X events
    while (XPending (dpy)) {
      XNextEvent(dpy, &event);
      switch (event.type) {
        case Expose: panel.reDraw (); break;
        case ButtonPress: {
          if (event.xbutton.button == Button1) {
            panel.processClick (event.xbutton.x, event.xbutton.y,
                event.xbutton.state & ShiftMask);
          }
          else if (event.xbutton.button == Button3) {
            if (requestUpdatedData ()) {
              // Request allowed, indicate this to the user
              panel.getStatusPainter ().setErrorText ("UPDATING.");
              panel.reDraw ();
            }
          }
          break;
        }
        default: break;
      }
    }

    // Check usage data
    fetcher->processNetworkEvent ();
  }
  cout << endl;

  fdSet.removeFDHolder (xfd);

  fetcher.reset (); // Must destroy this object before fdSet goes out of scope!

  return 0;
}


void printBanner () {
  cout << "wmium " << VERSION << " - Copyright(C) 2002-2005, Johny Mattsson\n";
  cout << "This software is licenced under the GNU General Public Licence ";
  cout << "version 2.\n\n";
}

void printHelp () {
  cout << "Available command line options:\n";
  cout << " -q\t\t\tQuiet mode (no copyright banner printed)\n";
  cout << " -d\t\t\tUse alternate date format (mm/dd)\n";
  cout << " -p <s,g>\t\tSet initial panel (status/graph)\n";
  cout << " -g <w,m,q,y>\t\tSet initial graph mode (week/month/quarter/year)\n";
  cout << " -f <filename>\t\tUse <filename> instead of ~/.wmiumrc\n";
  cout << " -s\t\t\tSnoop/debug mode, dumps received data to stderr\n";
  cout << " -h\t\t\tPrint this help\n\n";
}

void waitForMapNotify () {
  XEvent e;
  for(;;) {
    XNextEvent(dpy, &e);
    if (e.type == MapNotify) break;
  }
}

void scheduleTimer () {
  itimerval timer;
  timer.it_interval.tv_sec = 60 * 60 + 60; // One hour, one minute
  timer.it_interval.tv_usec = 0;
  timer.it_value.tv_sec = timer.it_interval.tv_sec;
  timer.it_value.tv_usec = timer.it_interval.tv_usec;

  int ret = setitimer (ITIMER_REAL, &timer, 0);
  if (ret != 0) {
    cerr << "Error: unable to set timer. No updates will happen!\n";
  }
}

void onTerm (int) {
  timeToQuit = true;
}

void onAlarm (int) {
  // queue up usage check
  if (fetcher.get ()) {
    fetcher->queueUsageCheck ();
    fetcher->queueHistoryCheck ();
  }
}

bool requestUpdatedData () {
  static time_t lastManualUpdate = 0;

  double minWaitBetweenUpdates = 60 * 5; // 5 minutes between manual updates

  if (difftime (time (0), lastManualUpdate) >= minWaitBetweenUpdates) {
    if (fetcher.get ()) {
      fetcher->queueUsageCheck (true);
      fetcher->queueHistoryCheck (true);
      fetcher->queueSpeedCheck (true);
    }

    lastManualUpdate = time (0);

    return true;
  }
  else return false;
}

